"""
drugstone.scripts.task_id

This module implements the class TaskId.

:copyright: 2022 Institute for Computational Systems Biology by Prof. Dr. Jan Baumbach
:author: Ugur Turhan
"""


class TaskId:
    """Creates unique ids.

    The ids are just integers, starting with 1 and counting up.
    """

    __u_id = 0

    @classmethod
    def get(cls) -> str:
        cls.__u_id += 1
        return str(cls.__u_id)





def add_edges_to_genes(
        genes: list,
        edges: list, ) -> dict:

    for gene in genes:
        if "netexId" in gene:
            netex_edges = [n["proteinB"] for n in edges if gene["netexId"] == n["proteinA"]]
            symbol_edges = []
            for e in netex_edges:
                for g in genes:
                    if "symbol" in g and "netexId" in g and e == g["netexId"]:
                        symbol_edges.append(g["symbol"])
            gene["has_edges_to"] = symbol_edges
        else:
            gene["has_edges_to"] = []

    result = {"drugs": {}, "genes": {}}

    for gene in genes:
        gene.pop("netexId", None)
        result["genes"][gene["id"]] = gene

    return result



"""
drugstone.scripts.normalize_task_parameter

This module implements the normalize_task_parameter function.

:copyright: 2022 Institute for Computational Systems Biology by Prof. Dr. Jan Baumbach
:author: Ugur Turhan
"""


import warnings
from typing import Dict
from .task_id import TaskId
from .constants.task_parameter import TaskParameter


def normalize_task_parameter(user_params: dict, seeds: list) -> dict:
    """Normalizes the parameter dictionary from the user."""

    normalized_params: Dict[str, any] = {
        "algorithm": "trustrank",
        "target": "drug",
        "parameters": {
            "target": "drug",
            "ppi_dataset": "STRING",
            "pdi_dataset": "drugbank",
            "result_size": 20,
            "config": {"identifier": "symbol"},
        }
    }

    for key, value in user_params.items():
        if key == "algorithm" or key == "algorithms":
            if value in TaskParameter.AlgorithmValues.ALGORITHM_VALUES:
                normalized_params["algorithm"] = value
            else:
                warnings.warn(str(value) + "-algorithm is not known to Drugstone!"
                              + " The algorithm is changed to "
                              + normalized_params["algorithm"] + "!    ")
        elif key == "target":
            if value in TaskParameter.TargetValues.TARGET_VALUES:
                normalized_params["target"] = value
                normalized_params["parameters"]["target"] = value
            else:
                warnings.warn("The target: " + str(value) + " is not known to Drugstone!"
                              + " The target is changed to "
                              + normalized_params["target"] + "!    ")
        elif key == "identifier":
            if value in TaskParameter.IdentifierValues.IDENTIFIER_VALUES:
                normalized_params["parameters"]["config"]["identifier"] = value
            else:
                warnings.warn("The identifier: " + str(value) + " is not known to Drugstone!"
                              + " The identifier is changed to "
                              + normalized_params["parameters"]["config"]["identifier"] + "!    ")
        elif key == "ppi_dataset":
            if value.lower() in TaskParameter.PpiValues.PPI_VALUES:
                normalized_params["parameters"]["ppi_dataset"] = value
            else:
                warnings.warn("The PPI-dataset: " + str(value) + " is not known to Drugstone!"
                              + " The PPI-dataset is changed to "
                              + normalized_params["parameters"]["ppi_dataset"] + "!    ")
        elif key == "pdi_dataset":
            if value.lower() in TaskParameter.PdiValues.PDI_VALUES:
                normalized_params["parameters"]["pdi_dataset"] = value
            else:
                warnings.warn("The PDI-dataset: " + str(value) + " is not known to Drugstone!"
                              + " The PDI-dataset is changed to "
                              + normalized_params["parameters"]["pdi_dataset"] + "!    ")
        elif key == "result_size":
            if isinstance(value, int):
                normalized_params["parameters"]["result_size"] = value
            else:
                warnings.warn("Invalid result_size: " + str(value) + ", has to be an integer!"
                              + " The result_size is changed to "
                              + str(normalized_params["parameters"]["result_size"]) + "!    ")
        else:
            normalized_params["parameters"][key] = value

    if normalized_params["algorithm"] == "keypathwayminer" and "k" not in normalized_params["parameters"]:
        normalized_params["parameters"]["k"] = 5
    if normalized_params["target"] == "drug-target" and normalized_params["algorithm"] == "proximity":
        warnings.warn("Network Proximity is not capable for Drug-Search!"
                      + " Drug-Search algorithm is changed to TrustRank!    ")
        normalized_params["algorithm"] = "trustrank"
    normalized_params["parameters"]["seeds"] = seeds
    normalized_params["parameters"]["input_network"] = {"nodes": [], "edges": []}
    alg = normalized_params["algorithm"]
    if "has_duplicate_algorithms" in user_params:
        if user_params["has_duplicate_algorithms"]:
            task_id = alg + "-" + TaskId.get()
            normalized_params["parameters"]["task_id"] = task_id
        else:
            normalized_params["parameters"]["task_id"] = alg
    else:
        normalized_params["parameters"]["task_id"] = alg
    return normalized_params



"""
drugstone.task.task

This module implements the class :class:`Task` for the drugstone API.

:copyright: 2022 Institute for Computational Systems Biology by Prof. Dr. Jan Baumbach
:author: Ugur Turhan
"""

from .task_result import TaskResult


class Task:
    """Represents a task.

    get_result() -> :class:`TaskResult`:
        Returns a :class:`TaskResult` for the result of the task.

    get_info() -> dict:
        Returns a dict with information about the task.

    get_parameters() -> dict:
        Returns a dict with the parameters of the task.
    """

    def __init__(self,
                 result: dict = None,
                 raw_data: dict = dict({}),
                 info: dict = dict({}),
                 params: dict = dict({})) -> None:
        self.__info = info
        self.__params = params
        self.__raw_data = raw_data
        if result is None:
            self.__result = dict({"drugs": {}, "genes": {}})
        else:
            self.__result = result

    def get_result(self) -> TaskResult:
        """Returns a :class:`TaskResult` for the result of the task."""

        return TaskResult(drugs=self.__result["drugs"],
                          genes=self.__result["genes"],
                          raw_data=self.__raw_data)

    def get_info(self) -> dict:
        """Returns a dict with information about the task."""

        return self.__info

    def get_parameters(self) -> dict:
        """Returns a dict with the parameters of the task."""

        return self.__params





def merge_results(first: dict, second: dict) -> dict:

    first_drugs = first["drugs"] if "drugs" in first else {}
    second_drugs = second["drugs"] if "drugs" in second else {}

    first_genes = first["genes"] if "genes" in first else {}
    second_genes = second["genes"] if "genes" in second else {}

    for drug, details in second_drugs.items():
        if drug in first_drugs:
            first_drugs[drug] = {**first_drugs[drug], **second_drugs[drug]}
        else:
            first_drugs[drug] = details

    for gene, details in second_genes.items():
        if gene in first_genes:
            for k, v in details.items():
                if k in first_genes[gene]:
                    if isinstance(v, list):
                        new_v = [*first_genes[gene][k], *v]
                        first_genes[gene][k] = new_v
                else:
                    first_genes[gene][k] = v
        else:
            first_genes[gene] = details

    return {"drugs": first_drugs, "genes": first_genes}
